import requests, json

from ragnar.cache import method_cache
from ragnar import log


class Api:

    base_url = "https://www.devrant.io/api/"

    def __init__(self, username, password):
        self.username = username
        self.password = password
        self.auth = None

    def post_comment(self, rant_id, text):
        response = requests.post(
            self.base_url + "devrant/rants/" + str(rant_id) + "/comments",
            data={
                "app": 3,
                "user_id": self.auth["user_id"],
                "token_id": self.auth["token_id"],
                "token_key": self.auth["token_key"],
                "comment": text,
            },
        )
        return response.json()

    @method_cache
    def login(self):
        log.info("Logged in as {}".format(self.username))
        rawdata = requests.post(
            self.base_url + "users/auth-token",
            data={"username": self.username, "password": self.password, "app": 3},
        )
        rawdata = rawdata.json()
        if not rawdata["success"]:
            self.auth = None
        else:
            self.auth = {
                "token_id": rawdata["auth_token"]["id"],
                "token_key": rawdata["auth_token"]["key"],
                "user_id": rawdata["auth_token"]["user_id"],
            }
        return self.auth

    @method_cache
    def get_profile(self, id_):
        url = self.base_url + "users/" + str(id_)
        params = {
            "app": 3,
        }
        response = requests.get(url, params)
        return json.loads(response.text)["profile"]

    def get_search(self, term):
        url = self.base_url + "devrant/search"
        params = {"app": 3, "term": term}
        response = requests.get(url, params, timeout=5)
        obj = json.loads(response.text)
        return obj

    def post_rant_vote(self, id_, vote):
        response = requests.post(
            self.base_url + "devrant/rants/" + str(id_) + "/vote",
            data={
                "app": 3,
                "user_id": self.auth["user_id"],
                "token_id": self.auth["token_id"],
                "token_key": self.auth["token_key"],
                "vote": str(vote),
            },
        )
        return json.loads(response.text)

    def get_rant(self, rant_id):
        url = self.base_url + "devrant/rants/" + str(rant_id)
        params = {
            "app": 3,
        }
        response = requests.get(url, params, timeout=5)
        return json.loads(response.text)

    def get_rants(self, sort, limit, skip):
        url = self.base_url + "devrant/rants"
        params = {"app": 3, "sort": sort, "limit": limit, "skip": skip}

        response = requests.get(url, params, timeout=5)
        return json.loads(response.text)["rants"]

    @method_cache
    def get_user_id(self, name):
        url = self.base_url + "get-user-id"
        params = {"app": 3, "username": name}
        response = requests.get(url, params)
        return json.loads(response.text).get("user_id", None)
