from ragnar.api import Api
import time
import random
from ragnar.cache import method_cache
import re
from ragnar import log
import json


class Bot:

    def __init__(self, username, password):
        self.username = username
        self.password = password
        self.name = self.username.split("@")[0]
        self.rant_history = []
        self.amount_of_rants_to_check = 5
        names = {
            "no-spam": "anna",
            "no-spam1": "ira",
            "no-spam2": "katya",
            "no-spam3": "nastya",
            "no-spam4": "vira",
        }
        self.name = names.get(self.name, "everyone")
        self.mark_text = "I am {} and downvoted you because you are spam. Your message will be removed from this community site due too much downvotes. See my profile for more information. Read my source code mentioned on my profile to see what you did wrong. Should be no problem for a developer. Have a nice day!\n\n@retoor: are you proud?".format(
            self.name
        )
        self.auth = None
        self.triggers = [
            "$",
            "crypto",
            "hacker",
            "recovery",
            {"regex": r"\([+,(,0-9,),-]{7,}"},
            "money",
            "landscape",
            "cyber",
            "recover",
            "trust",
            "bitcoin",
            "wizard",
        ]
        self.api = Api(username=self.username, password=self.password)

    def rsleepii(self):
        time.sleep(random.randint(1, 3))

    @method_cache
    def login(self):
        self.rsleepii()
        self.auth = self.api.login()
        if not self.auth:
            log.error("Authentication for {} failed.".format(self.username))
            raise Exception("Login error")
        log.info("Authentication succesful for {}.".format(self.username))

    def clean_rant_text(self, rant_text):
        return rant_text.replace(" ", "").lower()

    @method_cache
    def is_sus_rant(self, rant_id, rant_text):
        clean_text = self.clean_rant_text(rant_text)
        for trigger in self.triggers:
            if type(trigger) == dict:
                if trigger.get("regex"):
                    regex = trigger["regex"]
                    if re.search(regex, clean_text):
                        log.info("Regex trigger {} matched!".format(regex))
                        return True
            elif trigger in clean_text:
                log.info("Trigger {} matched!".format(trigger))
                return True
        return False

    def is_flagged_as_sus(self, rant_id, num_comments):
        if not num_comments:
            return False
        self.rsleepii()
        rant = self.api.get_rant(rant_id)
        for comment in rant.get("comments", []):
            if self.mark_text in comment.get("body", ""):
                return True
        return False

    @method_cache
    def is_user_sus(self, username):
        user_id = self.api.get_user_id(username)
        profile = self.api.get_profile(user_id)
        score = profile["score"]
        if score < 5:
            log.warning(
                "User {} is sus with his score of only {}.".format(username, score)
            )
            return True
        else:
            return False

    def mark_as_sus(self, rant):
        self.rsleepii()
        self.api.post_comment(rant["id"], self.mark_text)

    def fight(self):
        self.rsleepii()
        rants = self.api.get_rants("recent", self.amount_of_rants_to_check, 0)
        for rant in rants:
            if rant["id"] in self.rant_history:
                log.debug("{}: Already checked rant {}.".format(self.name, rant["id"]))
                continue
            else:
                self.rant_history.append(rant["id"])
            if not self.is_user_sus(rant["user_username"]):
                log.info(
                    "{}: User {} is trusted.".format(self.name, rant["user_username"])
                )
                continue
            if not self.is_sus_rant(rant["id"], rant["text"]):
                log.info(
                    "{}: Rant by {} is not sus.".format(
                        self.name, rant["user_username"]
                    )
                )
                continue
            if self.is_flagged_as_sus(rant["id"], rant.get("num_comments")):
                continue
            log.warning(
                "{}: Rant is not {} flagged as sus yet.".format(
                    self.name, rant["user_username"]
                )
            )
            log.warning(
                "{}: Flagging rant by {} as sus.".format(
                    self.name, rant["user_username"]
                )
            )
            self.mark_as_sus(rant)
            self.down_vote_rant(rant)

    def down_vote_rant(self, rant):
        log.warning("Downvoting rant by {}.".format(rant["user_username"]))
        log.debug(json.loads(self.api.post_rant_vote(rant["id"], -1)))
